const { omit, get, last } = require('lodash')

const config = require('config')

const statuses = config.get('statuses')

const keysToOmit = ['email', 'id']
const publicStatusesPermissions = ['author', 'reviewer']
const authorAllowedStatuses = ['revisionRequested', 'rejected', 'accepted']

const parseAuthorsData = (coll, matchingCollPerm) => {
  if (['reviewer'].includes(matchingCollPerm.permission)) {
    coll.authors = coll.authors.map(a => omit(a, ['email']))
  }
}

const setPublicStatuses = (coll, matchingCollPerm) => {
  const status = get(coll, 'status', 'draft')
  // coll.visibleStatus = statuses[status].public
  if (publicStatusesPermissions.includes(matchingCollPerm.permission)) {
    coll.visibleStatus = statuses[status].public
  }
}

const filterRefusedInvitations = (coll, user) => {
  const matchingInv = coll.invitations.find(inv => inv.userId === user.id)
  if (matchingInv === undefined) return null
  if (matchingInv.hasAnswer === true && !matchingInv.isAccepted) return null
  return coll
}

const filterObjectData = (
  collectionsPermissions = [],
  object = {},
  user = {},
) => {
  if (object.type === 'fragment') {
    const matchingCollPerm = collectionsPermissions.find(
      collPerm => object.id === collPerm.fragmentId,
    )
    if (matchingCollPerm === undefined) return null
    if (['reviewer'].includes(matchingCollPerm.permission)) {
      object.files = omit(object.files, ['coverLetter'])
      if (object.recommendations)
        object.recommendations = object.recommendations.filter(
          rec => rec.userId === user.id,
        )
    }
    parseAuthorsData(object, matchingCollPerm)
    if (['reviewer', 'handlingEditor'].includes(matchingCollPerm.permission)) {
      return filterRefusedInvitations(object, user)
    }
    return object
  }
  const matchingCollPerm = collectionsPermissions.find(
    collPerm => object.id === collPerm.id,
  )
  if (matchingCollPerm === undefined) return null
  setPublicStatuses(object, matchingCollPerm)

  return object
}

const getTeamsByPermissions = async (
  teamIds = [],
  permissions = [],
  TeamModel,
) =>
  (await Promise.all(
    teamIds.map(async teamId => {
      const team = await TeamModel.find(teamId)
      if (!permissions.includes(team.teamType.permissions)) {
        return null
      }
      return team
    }),
  )).filter(Boolean)

const heIsInvitedToFragment = async ({ user, Team, collectionId }) =>
  (await getTeamsByPermissions(user.teams, ['handlingEditor'], Team)).some(
    // user is a member of the team with access to the fragment's parent collection
    t => t.members.includes(user.id) && t.object.id === collectionId,
  )

const getUserPermissions = async ({
  user,
  Team,
  mapFn = t => ({
    objectId: t.object.id,
    objectType: t.object.type,
    role: t.teamType.permissions,
  }),
}) =>
  (await Promise.all(user.teams.map(teamId => Team.find(teamId)))).map(mapFn)

const isOwner = ({ user: { id }, object }) => {
  if (object.owners.includes(id)) return true
  return !!object.owners.find(own => own.id === id)
}

const hasPermissionForObject = async ({ user, object, Team, roles = [] }) => {
  const userPermissions = await getUserPermissions({
    user,
    Team,
  })

  return !!userPermissions.find(p => {
    const hasObject =
      p.objectId === get(object, 'fragment.id') ||
      p.objectId === get(object, 'fragment.collectionId')
    if (roles.length > 0) {
      return hasObject && roles.includes(p.role)
    }
    return hasObject
  })
}

const isHandlingEditor = ({ user, object }) =>
  get(object, 'collection.handlingEditor.id') === user.id

const isInDraft = fragment => !get(fragment, 'submitted')

const hasFragmentInDraft = async ({ object, Fragment }) => {
  const lastFragmentId = last(get(object, 'fragments'))
  const fragment = await Fragment.find(lastFragmentId)
  return isInDraft(fragment)
}

const filterAuthorRecommendations = (recommendations, status, isLast) => {
  const canViewRecommendations = authorAllowedStatuses.includes(status)
  if (canViewRecommendations || !isLast) {
    return recommendations.map(r => ({
      ...r,
      comments: r.comments ? r.comments.filter(c => c.public) : [],
    }))
  }
  return []
}

const stripeCollectionByRole = (coll = {}, role = '') => {
  if (role === 'author') {
    const { handlingEditor } = coll

    if (!authorAllowedStatuses.includes(coll.status)) {
      return {
        ...coll,
        handlingEditor: handlingEditor &&
          handlingEditor.isAccepted && {
            ...omit(handlingEditor, keysToOmit),
            name: 'Assigned',
          },
      }
    }
  }
  return coll
}

const stripeFragmentByRole = ({
  fragment = {},
  role = '',
  status = 'draft',
  user = {},
  isLast = false,
}) => {
  const { recommendations, files, authors } = fragment
  switch (role) {
    case 'author':
      return {
        ...fragment,
        recommendations: recommendations
          ? filterAuthorRecommendations(recommendations, status, isLast)
          : [],
      }
    case 'reviewer':
      return {
        ...fragment,
        files: omit(files, ['coverLetter']),
        authors: authors.map(a => omit(a, ['email'])),
        recommendations: recommendations
          ? recommendations.filter(r => r.userId === user.id)
          : [],
      }
    default:
      return fragment
  }
}

const sensitiveUserProperties = ['passwordHash', 'accessTokens']

const getUsersList = async ({ UserModel, user }) => {
  const users = await UserModel.all()
  const omitUserProperties = [...sensitiveUserProperties]
  if (!user.editorInChief && !user.admin && !user.handlingEditor) {
    omitUserProperties.push('email', 'username')
  }

  const parsedUsers = users.map(foundUser =>
    omit(foundUser, omitUserProperties),
  )

  return parsedUsers
}

const parseUser = ({ user }) => omit(user, sensitiveUserProperties)

module.exports = {
  filterObjectData,
  parseAuthorsData,
  setPublicStatuses,
  getTeamsByPermissions,
  filterRefusedInvitations,
  isOwner,
  isHandlingEditor,
  getUserPermissions,
  heIsInvitedToFragment,
  hasPermissionForObject,
  isInDraft,
  hasFragmentInDraft,
  stripeCollectionByRole,
  stripeFragmentByRole,
  getUsersList,
  parseUser,
}
